@@ -3,9 +3,9 @@
  * imp.php - performs an AJAX-requested action and returns the DIMP-specific
  * JSON object
  *
- * $Horde: dimp/imp.php,v 1.194.2.21 2008/05/13 18:22:45 slusarz Exp $
+ * $Horde: dimp/imp.php,v 1.194.2.37 2009/01/06 15:22:36 jan Exp $
  *
- * Copyright 2005-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2005-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -71,23 +71,28 @@ function _threadUidChanged($folder, $indices)
 function _getListMessages($folder, $change)
 {
     $args = array(
-        'buffer_size' => intval(Util::getPost('buffer_size')),
         'cacheid' => Util::getPost('cacheid'),
         'filter' => Util::getPost('filter'),
         'folder' => $folder,
-        'nearing' => Util::getPost('nearing'),
-        'page_size' => intval(Util::getPost('page_size')),
         'searchfolder' => Util::getPost('searchfolder'),
         'searchmsg' => Util::getPost('searchmsg'),
     );
 
     $search = Util::getPost('search');
     if (!empty($search)) {
-        require_once 'Horde/Serialize.php';
         $search = Horde_Serialize::unserialize($search, SERIALIZE_JSON);
-        $args['uid'] = $search->imapuid;
+        $args += array(
+            'search_uid' => $search->imapuid,
+            'search_view' => $search->view,
+            'search_before' => intval(Util::getPost('search_before')),
+            'search_after' => intval(Util::getPost('search_after'))
+        );
     } else {
-        $args['offset'] = intval(Util::getPost('offset'));
+        $args += array(
+            'slice_rownum' => intval(Util::getPost('rownum')),
+            'slice_start' => intval(Util::getPost('slice_start')),
+            'slice_end' => intval(Util::getPost('slice_end'))
+        );
     }
 
     require_once DIMP_BASE . '/lib/Views/ListMessages.php';
@@ -95,10 +100,15 @@ function _getListMessages($folder, $change)
     $res = $list_msg->ListMessages($args);
     // TODO: This can potentially be optimized for arrival time sort - if the
     // cache ID changes, we know the changes must occur at end of mailbox.
-    if (Util::getPost('update') || $change) {
+    if (Util::getPost('purge') || $change) {
         $res->update = true;
     }
-    $res->request_id = Util::getPost('request_id');
+
+    $req_id = Util::getPost('request_id');
+    if (!is_null($req_id)) {
+        $res->request_id = $req_id;
+    }
+
     return $res;
 }
 
@@ -115,11 +125,28 @@ function _getPollInformation($mbox)
     $elt = $imptree->get($mbox);
     if ($imptree->isPolled($elt)) {
         $info = $imptree->getElementInfo($mbox);
-        return array($mbox => array('u' => isset($info['unseen']) ? $info['unseen'] : 0, 'n' => isset($info['newmsg']) ? $info['newmsg'] : 0));
+        return array($mbox => isset($info['unseen']) ? $info['unseen'] : 0);
     }
     return array();
 }
 
+function _getQuota()
+{
+    if (isset($_SESSION['imp']['quota']) &&
+        is_array($_SESSION['imp']['quota'])) {
+        $quotadata = IMP::quotaData(false);
+        if (!empty($quotadata)) {
+            require_once IMP_BASE . '/lib/Template.php';
+            $t = new IMP_Template(DIMP_TEMPLATES . '/imp/');
+            $t->set('img', Horde::img('quotauncover.gif', '', array('width' => 99 - $quotadata['percent'], 'height' => '10'), $GLOBALS['registry']->getImageDir('dimp')));
+            $t->set('msg', $quotadata['message']);
+            return $t->fetch('quota.html');
+        }
+    }
+
+    return null;
+}
+
 // Need to load Util:: to give us access to Util::getPathInfo().
 if (!defined('HORDE_BASE')) {
     define('HORDE_BASE', dirname(__FILE__) . '/..');
@@ -135,20 +162,22 @@ if (empty($action)) {
 
 // The following actions do not need write access to the session and
 // should be opened read-only for performance reasons.
-if (in_array($action, array('chunkContent', 'ShowPreview', 'Html2Text', 'Text2Html', 'GetReplyData'))) {
+if (in_array($action, array('chunkContent', 'Html2Text', 'Text2Html', 'GetReplyData'))) {
     $session_control = 'readonly';
 }
 
 $authentication = 'none';
+$dimp_logout = ($action == 'LogOut');
 $load_imp = true;
 $session_timeout = 'json';
 @define('AUTH_HANDLER', true);
 @define('DIMP_BASE', dirname(__FILE__));
 require_once DIMP_BASE . '/lib/base.php';
+require_once 'Horde/Serialize.php';
 
 // Process common request variables.
 $folder = Util::getPost('folder');
-$indices = IMP::parseIndicesList(Util::getPost('uid'));
+$indices = IMP::parseIndicesList(Horde_Serialize::unserialize(Util::getPost('uid'), SERIALIZE_JSON));
 $cacheid = Util::getPost('cacheid');
 
 // Open an output buffer to ensure that we catch errors that might break JSON
@@ -159,16 +188,10 @@ $result = false;
 
 switch ($action) {
 case 'CreateFolder':
-case 'CreateSubfolder':
     if (empty($folder)) {
         break;
     }
 
-    $parent = Util::getPost('parent');
-    if (($action == 'CreateSubfolder') && !$parent) {
-        break;
-    }
-
     require_once IMP_BASE . '/lib/IMAP/Tree.php';
     $imptree = &IMP_Tree::singleton();
     $imptree->eltDiffStart();
@@ -177,7 +200,7 @@ case 'CreateSubfolder':
     $imp_folder = &IMP_Folder::singleton();
 
     $new = String::convertCharset($folder, NLS::getCharset(), 'UTF7-IMAP');
-    $new = $imptree->createMailboxName($parent, $new);
+    $new = $imptree->createMailboxName(Util::getPost('parent'), $new);
     if (is_a($new, 'PEAR_Error')) {
         $notification->push($new, 'horde.error');
         $result = false;
@@ -221,26 +244,20 @@ case 'RenameFolder':
     require_once IMP_BASE . '/lib/Folder.php';
     $imp_folder = &IMP_Folder::singleton();
 
-    if (!empty($new_parent)) {
-        $ns = IMP::getNamespace($new_parent);
-        $new = $new_parent . $ns['delimiter'] . $new;
+    $new = $imptree->createMailboxName($new_parent, $new);
+    if (is_a($new, 'PEAR_Error')) {
+        $notification->push($new, 'horde.error');
+        $result = false;
     } else {
-        $ns = IMP::getNamespace($old);
-        $lastDelimiter = strrpos($old, $ns['delimiter']);
-        if ($lastDelimiter !== false) {
-            $new = substr($old, 0, $lastDelimiter + 1) . $new;
+        require_once 'Horde/String.php';
+        $new = String::convertCharset($new, NLS::getCharset(), 'UTF7-IMAP');
+        if ($old != $new) {
+            $result = $imp_folder->rename($old, $new);
+            if ($result) {
+                $result = DIMP::getFolderResponse($imptree);
+            }
         }
     }
-
-    require_once 'Horde/String.php';
-    $new = String::convertCharset($new, NLS::getCharset(), 'UTF7-IMAP');
-    if ($old == $new) {
-        break;
-    }
-    $result = $imp_folder->rename($old, $new);
-    if ($result) {
-        $result = DIMP::getFolderResponse($imptree);
-    }
     break;
 
 case 'EmptyFolder':
@@ -272,50 +289,50 @@ case 'MarkFolderUnseen':
 
         $poll = _getPollInformation($folder);
         if (!empty($poll)) {
-            $result->u = $poll[$folder]['u'];
-            $result->n = $poll[$folder]['n'];
+            $result->poll = array($folder => $poll[$folder]['u']);
         }
     }
     break;
 
 case 'ListFolders':
     require_once IMP_BASE . '/lib/IMAP/Tree.php';
-
-    /* Initialize the IMP_Tree object. */
     $imptree = &IMP_Tree::singleton();
-    $mask = IMPTREE_FLIST_CONTAINER;
-    if (!$prefs->getValue('subscribe')) {
-        $mask |= IMPTREE_FLIST_UNSUB;
-    }
+    $result = DIMP::getFolderResponse($imptree, array('a' => $imptree->folderList(IMPTREE_FLIST_CONTAINER), 'c' => array(), 'd' => array()));
 
-    $result = DIMP::getFolderResponse($imptree, array('a' => $imptree->folderList($mask), 'c' => array(), 'd' => array()));
+    $quota = _getQuota();
+    if (!is_null($quota)) {
+        $result['quota'] = $quota;
+    }
     break;
 
 case 'PollFolders':
     $result = new stdClass;
 
-    if ($action == 'PollFolders') {
-        require_once IMP_BASE . '/lib/Fetchmail.php';
-        $fm_account = new IMP_Fetchmail_Account();
-        $fm_count = $fm_account->count();
-        if (!empty($fm_count)) {
-            IMP_Fetchmail::fetchMail(range(0, $fm_count - 1));
-        }
+    require_once IMP_BASE . '/lib/Fetchmail.php';
+    $fm_account = new IMP_Fetchmail_Account();
+    $fm_count = $fm_account->count();
+    if (!empty($fm_count)) {
+        IMP_Fetchmail::fetchMail(range(0, $fm_count - 1));
+    }
 
-        require_once IMP_BASE . '/lib/IMAP/Tree.php';
-        $imptree = &IMP_Tree::singleton();
+    require_once IMP_BASE . '/lib/IMAP/Tree.php';
+    $imptree = &IMP_Tree::singleton();
 
-        $result->poll = array();
-        foreach ($imptree->getPollList(true) as $val) {
-            if ($info = $imptree->getElementInfo($val)) {
-                $result->poll[$val] = array('u' => $info['unseen'], 'n' => $info['newmsg']);
-            }
+    $result->poll = array();
+    foreach ($imptree->getPollList(true) as $val) {
+        if ($info = $imptree->getElementInfo($val)) {
+            $result->poll[$val] = $info['unseen'];
         }
     }
 
     if (!empty($folder) && _changed($folder, $cacheid)) {
         $result->viewport = _getListMessages($folder, true);
     }
+
+    $quota = _getQuota();
+    if (!is_null($quota)) {
+        $result->quota = $quota;
+    }
     break;
 
 case 'ListMessages':
@@ -478,7 +495,6 @@ case 'Whitelist':
     }
     break;
 
-// Needs: UID of message (uid)
 case 'ShowPreview':
     if (count($indices) != 1) {
         break;
@@ -682,7 +698,7 @@ case 'ModifyPollFolder':
     if ($add) {
         $imptree->addPollList($folder);
         if ($info = $imptree->getElementInfo($folder)) {
-            $result->poll = array('u' => $info['unseen'], 'n' => $info['newmsg']);
+            $result->poll = array($folder => $info['unseen']);
         }
     } else {
         $imptree->removePollList($folder);
@@ -708,23 +724,6 @@ case 'SendMDN':
     $imp_ui = new IMP_UI_Message();
     $imp_ui->MDNCheck($ob->header, true);
     break;
-
-case 'CheckQuota':
-    $quotadata = IMP::quotaData(false);
-    if (!empty($quotadata)) {
-        require_once IMP_BASE . '/lib/Template.php';
-        $t = new IMP_Template(DIMP_TEMPLATES . '/imp/');
-        $t->set('img', Horde::img('quotauncover.gif', '', array('width' => 99 - $quotadata['percent'], 'height' => '10'), $registry->getImageDir('dimp')));
-        $t->set('msg', $quotadata['message']);
-
-        $result = new stdClass;
-        $result->quota = $t->fetch('quota.html');
-    }
-    break;
-
-case 'LogOut':
-    IMP::redirect(str_replace('&amp;', '&', IMP::getLogoutURL()));
-    break;
 }
 
 // Clear the output buffer that we started above, and log any unexpected
@@ -736,4 +735,4 @@ if ($errors) {
 }
 
 // Send the final result.
-IMP::sendHTTPResponse(DIMP::getNotificationResponse($result), 'json');
+IMP::sendHTTPResponse(DIMP::prepareResponse($result), 'json');
