@@ -2,9 +2,9 @@
  * DimpCore.js - Dimp UI application logic.
  * NOTE: ContextSensitive.js must be loaded before this file.
  *
- * $Horde: dimp/js/src/DimpCore.js,v 1.369.2.37 2008/05/20 16:34:13 slusarz Exp $
+ * $Horde: dimp/js/src/DimpCore.js,v 1.369.2.61 2009/01/06 15:22:37 jan Exp $
  *
- * Copyright 2005-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2005-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -14,7 +14,7 @@
 var frames = { horde_main: true },
 
 /* DimpCore object. */
-    DimpCore = {
+DimpCore = {
     // Vars used and defaulting to null/false:
     //   DMenu, inAjaxCallback, is_logout, onDoActionComplete
     view_id: 1,
@@ -26,7 +26,7 @@ var frames = { horde_main: true },
     debug: function(label, e)
     {
         if (!this.is_logout && DIMP.conf.debug) {
-            alert(label + ': ' + Object.inspect(e));
+            alert(label + ': ' + (e instanceof Error ? e.name + '-' + e.message : Object.inspect(e)));
         }
     },
 
@@ -50,21 +50,30 @@ var frames = { horde_main: true },
      *  instead of sending the action to the IMP handler. */
     doAction: function(action, params, uids, callback, opts)
     {
-        opts = opts || {};
+        if (!this.doActionOpts) {
+            this.doActionOpts = {
+                onException: function(r, e) {
+                    this.debug('onException', e);
+                }.bind(this),
+                onFailure: function(t, o) {
+                    this.debug('onFailure', t);
+                }.bind(this)
+            };
+        };
+
+        opts = Object.extend(this.doActionOpts, opts || {});
         params = $H(params);
         action = action.startsWith('*')
             ? action.substring(1)
             : DIMP.conf.URI_IMP + '/' + action;
         if (uids && uids.size()) {
-            params.set('uid[]', uids);
+            params.set('uid', uids.toJSON());
         }
         if (DIMP.conf.SESSION_ID) {
             params.update(DIMP.conf.SESSION_ID.toQueryParams());
         }
         opts.parameters = params.toQueryString();
         opts.onComplete = function(t, o) { this.doActionComplete(t, callback); }.bind(this);
-        opts.onException = function(r, e) { this.debug('onException', e); }.bind(this);
-        opts.onFailure = function(t, o) { this.debug('onFailure', t) }.bind(this);
         new Ajax.Request(action, opts);
     },
 
@@ -84,6 +93,10 @@ var frames = { horde_main: true },
             }
         }
 
+        if (!r.msgs) {
+            r.msgs = [];
+        }
+
         if (error) {
             if (++this.server_error == 3) {
                 this.showNotifications([ { type: 'horde.error', message: DIMP.text.ajax_timeout } ]);
@@ -105,7 +118,7 @@ var frames = { horde_main: true },
         }
         this.server_error = 0;
 
-        if (r.msgs && r.msgs_auto) {
+        if (!r.msgs_noauto) {
             this.showNotifications(r.msgs);
         }
 
@@ -130,7 +143,7 @@ var frames = { horde_main: true },
         var alerts = $('alerts');
         if (!alerts) {
             alerts = new Element('DIV', { id: 'alerts' });
-            document.body.appendChild(alerts);
+            $(document.body).insert(alerts);
         }
 
         msgs.find(function(m) {
@@ -139,7 +152,6 @@ var frames = { horde_main: true },
                 this.is_logout = true;
                 this.redirect(DIMP.conf.timeout_url);
                 return true;
-                break;
 
             case 'horde.error':
             case 'horde.message':
@@ -150,13 +162,14 @@ var frames = { horde_main: true },
             case 'imp.redirect':
             case 'dimp.request':
             case 'dimp.sticky':
-                var clickdiv, div, fadeeffect, iefix, requestfunc;
+                var clickdiv, fadeeffect, iefix, requestfunc,
+                    div = new Element('DIV', { className: m.type.replace('.', '-') });
                 if ($w('dimp.request dimp.sticky').indexOf(m.type) != -1) {
-                    div = new Element('DIV', { className: m.type.replace('.', '-') }).update(m.message);
+                    div.update(m.message);
                 } else {
-                    div = new Element('DIV', { className: m.type.replace('.', '-') }).insert(m.message.unescapeHTML().unescapeHTML());
+                    div.insert(m.message.unescapeHTML().unescapeHTML());
                 }
-                alerts.appendChild(div);
+                alerts.insert(div);
 
                 // IE6 has a bug that does not allow the body of a div to be
                 // clicked to trigger an onclick event for that div (it only
@@ -168,11 +181,10 @@ var frames = { horde_main: true },
                 // overlay the div with a like sized div containing a clear
                 // gif, which tricks IE into the correct behavior.
                 if (DIMP.conf.is_ie6) {
-                    iefix = new Element('DIV', { className: 'ie6alertsfix' });
-                    iefix.clonePosition(div, { setLeft: false, setTop: false });
+                    iefix = new Element('DIV', { className: 'ie6alertsfix' }).clonePosition(div, { setLeft: false, setTop: false });
                     clickdiv = iefix;
-                    iefix.appendChild(div.remove());
-                    alerts.appendChild(iefix);
+                    iefix.insert(div.remove());
+                    alerts.insert(iefix);
                 } else {
                     clickdiv = div;
                 }
@@ -182,11 +194,14 @@ var frames = { horde_main: true },
                 clickdiv.observe('click', fadeeffect);
 
                 if ($w('horde.error dimp.request dimp.sticky').indexOf(m.type) == -1) {
-                    fadeeffect.delay(3);
+                    fadeeffect.delay(m.type == 'horde.warning' ? 10 : 5);
                 }
 
                 if (m.type == 'dimp.request') {
-                    requestfunc = function() { fadeeffect(); document.stopObserving('click', requestfunc) };
+                    requestfunc = function() {
+                        fadeeffect();
+                        document.stopObserving('click', requestfunc)
+                    };
                     document.observe('click', requestfunc);
                 }
             }
@@ -216,15 +231,16 @@ var frames = { horde_main: true },
     compose: function(type, args)
     {
         var url = DIMP.conf.compose_url;
+        args = args || {};
         if (type) {
-            url += (url.include('?') ? '&' : '?') + $H(args).merge({ type: type }).toQueryString();
+            args.type = type;
         }
-        this.popupWindow(url, '');
+        this.popupWindow(this.addURLParam(url, args), 'compose' + new Date().getTime());
     },
 
     popupWindow: function(url, name)
     {
-        if (!window.open(url, name, 'width=' + DIMP.conf.popup_width + ',height=' + DIMP.conf.popup_height + ',status=1,scrollbars=yes,resizable=yes')) {
+        if (!(window.open(url, name.replace(/\W/g, '_'), 'width=' + DIMP.conf.popup_width + ',height=' + DIMP.conf.popup_height + ',status=1,scrollbars=yes,resizable=yes'))) {
             this.showNotifications([ { type: 'horde.warning', message: DIMP.text.popup_block } ]);
         }
     },
@@ -252,6 +268,7 @@ var frames = { horde_main: true },
 
     redirect: function(url)
     {
+        url = this.addSID(url);
         if (parent.frames.horde_main) {
             parent.location = url;
         } else {
@@ -261,7 +278,7 @@ var frames = { horde_main: true },
 
     /* Add/remove mouse events on the fly.
      * Parameter: object with the following names - id, type, offset
-     *   (optional), left (optional), offsettype (optional), onShow (optional)
+     *   (optional), left (optional), onShow (optional)
      * Valid types:
      *   'message', 'draft'  --  Message list rows
      *   'container', 'special', 'folder'  --  Folders
@@ -272,19 +289,20 @@ var frames = { horde_main: true },
         this.DMenu.addElement(p.id, 'ctx_' + p.type, p);
     },
 
-    removeMouseEvents: function(id)
+    /* elt = DOM element */
+    removeMouseEvents: function(elt)
     {
-        this.DMenu.removeElement(id);
+        this.DMenu.removeElement($(elt).readAttribute('id'));
+        this.addGC(elt);
     },
 
     /* Add a popdown menu to a dimpactions button. */
-    addPopdown: function(bid, ctx, offsettype)
+    addPopdown: function(bid, ctx)
     {
         var bidelt = $(bid),
-            img = $($('popdown_img').cloneNode(false)).writeAttribute('id', bid + '_img').show(),
             parentelt = bidelt.up();
-        parentelt.appendChild(img);
-        this.addMouseEvents({ id: img.readAttribute('id'), type: ctx, offset: parentelt, left: true, offsettype: offsettype });
+        parentelt.insert($($('popdown_img').cloneNode(false)).writeAttribute('id', bid + '_img').show());
+        this.addMouseEvents({ id: bid + '_img', type: ctx, offset: parentelt, left: true });
     },
 
     /* Add dropdown menus to address links. */
@@ -292,15 +310,18 @@ var frames = { horde_main: true },
     {
         var acount = 0;
         $(id).select('.address').each(function(e) {
-            e.id = 'addr' + acount++;
-            e.observe('mouseover', function() { this.removeMouseEvents(e.id); this.addMouseEvents({ id: e.id, type: 'contacts', offset: e, left: true, offsettype: 'cumulative' }); }.bind(this));
+            e.writeAttribute({ id: 'addr' + acount++ });
+            e.observe('mouseover', function() { this.addMouseEvents({ id: e.id, type: 'contacts', offset: e, left: true }); }.bind(this));
+        }, this);
+        $('msgHeadersContent').select('.largeaddrlist').each(function(a) {
+           this.clickObserveHandler({ d: a, f: function() { a.up().down().toggle().next().toggle().next().toggle(); } });
         }, this);
     },
 
     /* Removes event handlers from address links. */
     removeAddressLinks: function(id)
     {
-        this.addGC($(id).select('.address'));
+        [ $(id).select('.address'), $(id).select('.largeaddrlist') ].flatten().compact().each(this.removeMouseEvents.bind(this));
     },
 
     /* Add event observers to message output.  Adds observers used in both
@@ -309,9 +330,6 @@ var frames = { horde_main: true },
     {
         var C = this.clickObserveHandler;
 
-        $('msgHeadersContent').select('.largeaddrlist').each(function(a) {
-            C({ d: a.down('a'), f: function(a) { toggleAddressesBlock(a.id.substring(3), a.getAttribute('addrs')); } });
-        });
         if ($('partlist')) {
             C({ d: $('partlist_col').up(), f: function() { $('partlist', 'partlist_col', 'partlist_exp').invoke('toggle'); } });
         }
@@ -321,37 +339,22 @@ var frames = { horde_main: true },
         if ($('msg_view_source')) {
             C({ d: $('msg_view_source'), f: function() { view(DIMP.conf.msg_source_link.unescapeHTML(), DIMP.conf.msg_index + '|' + DIMP.conf.msg_folder) }.bind(this) });
         }
-        C({ d: $('ctx_contacts_new'), f: function() { this.compose('new', { to: this.DMenu.element().readAttribute('address') }); }.bind(this) });
-        C({ d: $('ctx_contacts_add'), f: function() { this.doAction('AddContact', { name: this.DMenu.element().readAttribute('personal'), email: this.DMenu.element().readAttribute('email') }, null, true); }.bind(this) });
+        C({ d: $('ctx_contacts_new'), f: function() { this.compose('new', { to: this.DMenu.element().readAttribute('address') }); }.bind(this), ns: true });
+        C({ d: $('ctx_contacts_add'), f: function() { this.doAction('AddContact', { name: this.DMenu.element().readAttribute('personal'), email: this.DMenu.element().readAttribute('email') }, null, true); }.bind(this), ns: true });
     },
 
     /* Utility functions. */
-    toggleBlock: function(block, toggle, hidetxt, showtxt, style)
-    {
-        block.toggle();
-
-        var link = new Element('A', { className: 'widget', style: style || '' }).insert('[' + (block.visible() ? hidetxt : showtxt) + ']');
-        link.observe('click', this.toggleBlock.bind(this, block, toggle, hidetxt, showtxt, style));
-
-        if (toggle.firstChild) {
-            $(toggle.firstChild).remove();
-        }
-        toggle.appendChild(link);
-
-        return false;
-    },
-
     addGC: function(elt)
     {
         this.remove_gc = this.remove_gc.concat(elt);
     },
 
     // o: (object) Contains the following items:
-    //    'd' - (required) The DOM element
-    //    'f' - (required) The function to bind to the click event
-    //    'p' - (optional) If set, passes in the event object to the called
-    //                     function
-    //    's' - (optional) If set, stop the event's propogation
+    //    'd'  - (required) The DOM element
+    //    'f'  - (required) The function to bind to the click event
+    //    'ns' - (optional) If set, don't stop the event's propogation
+    //    'p'  - (optional) If set, passes in the event object to the called
+    //                      function
     clickObserveHandler: function(o)
     {
         return o.d.observe('click', DimpCore._clickFunc.curry(o));
@@ -360,9 +363,28 @@ var frames = { horde_main: true },
     _clickFunc: function(o, e)
     {
         o.p ? o.f(e) : o.f();
-        if (o.s) {
+        if (!o.ns) {
             e.stop();
         }
+    },
+
+    addSID: function(url)
+    {
+        if (!DIMP.conf.SESSION_ID) {
+            return url;
+        }
+        return this.addURLParam(url, DIMP.conf.SESSION_ID.toQueryParams());
+    },
+
+    addURLParam: function(url, params)
+    {
+        var q = url.indexOf('?');
+
+        if (q != -1) {
+            params = $H(url.toQueryParams()).merge(params).toObject();
+            url = url.substring(0, q);
+        }
+        return url + '?' + Object.toQueryString(params);
     }
 };
 
@@ -409,31 +431,29 @@ document.observe('dom:loaded', function() {
 Element.addMethods({
     setText: function(element, text)
     {
-        if (!(element = $(element))) {
-            return;
-        }
         var t = 0;
         $A(element.childNodes).each(function(node) {
             if (node.nodeType == 3) {
                 if (t++) {
-                    node.remove();
+                    Element.remove(node);
                 } else {
                     node.nodeValue = text;
                 }
             }
         });
+
+        if (!t) {
+            $(element).insert(text);
+        }
     },
 
     getText: function(element, recursive)
     {
-        if (!(element = $(element))) {
-            return;
-        }
         var text = '';
         $A(element.childNodes).each(function(node) {
             if (node.nodeType == 3) {
                 text += node.nodeValue;
-            } else if (recursive && node.childNodes) {
+            } else if (recursive && node.hasChildNodes()) {
                 text += $(node).getText(true);
             }
         });
@@ -459,17 +479,7 @@ Object.extend(String.prototype, {
     }
 });
 
-// Workaround for prototypejs bug (as of 1.6.0):
-//   http://dev.rubyonrails.org/ticket/10561
-DimpCore.isLeftClick = function(e) {
-    if (Prototype.Browser.IE) {
-        return (e.button == 0 && e.type != 'contextmenu');
-    } else {
-        return e.isLeftClick();
-    }
-};
-
-/** Functions overriding IMP/Horde JS functions. **/
+/** Functions overriding IMP/Horde/prototypejs JS functions. **/
 
 /* We need to replace the IMP javascript for this function with code that
  * calls the correct DIMP functions. */
@@ -478,15 +488,18 @@ function popup_imp(url, w, h, args)
     DimpCore.compose('new', args.toQueryParams().toObject());
 }
 
-/* Necessary includes for toggle divs. */
+/* Necessary includes for toggle divs. Needed for BC purposes. */
 function toggleQuoteBlock(id, lines)
 {
-    DimpCore.toggleBlock($('qb_' + id), $('qt_' + id), DIMP.text.hidetext, DIMP.text.showtext + ' - ' + lines + ' ' + DIMP.text.lines, 'fontSize:70%;');
-}
+    var block = $('qb_' + id).toggle(),
+        toggle = $('qt_' + id),
+        tmp = toggle.down();
 
-function toggleAddressesBlock(field, count)
-{
-    DimpCore.toggleBlock($('ab_' + field), $('at_' + field), DIMP.text.hideaddr, DIMP.text.showaddr + ' - ' + count + ' ' + DIMP.text.recips);
+        tmp = toggle.down();
+        if (tmp) {
+            tmp.remove();
+        }
+        toggle.insert(new Element('A', { className: 'widget togglequote' }).setStyle({ fontSize: '70%' }).insert('[' + (block.visible() ? DIMP.text.hidetext : DIMP.text.showtext + ' - ' + lines + ' ' + DIMP.text.lines) + ']').observe('click', toggleQuoteBlock.curry(id, lines)));
 }
 
 /* For attachment viewing to work, replaces the function from
@@ -495,3 +508,12 @@ function view(url, uniqid)
 {
     window.open(url, ++DimpCore.view_id + uniqid.replace(/\W/g, '_'), 'menubar=yes,toolbar=no,location=no,status=no,scrollbars=yes,resizable=yes');
 }
+
+/* document.viewport.getDimensions() is broken on Opera 9.5 in
+ * prototypejs 1.6.0.2 and totally prevents usage of ViewPort.
+ * Redefine here. */
+if (Prototype.Browser.Opera && opera.version() >= 9.5) {
+    document.viewport.getDimensions = function() {
+        return { width: document.documentElement['clientWidth'], height: document.documentElement['clientHeight'] };
+    };
+}
