@@ -1,9 +1,9 @@
 /**
  * compose.js - Javascript code used in the DIMP compose view.
  *
- * $Horde: dimp/js/src/compose.js,v 1.84.2.18 2008/05/20 16:34:13 slusarz Exp $
+ * $Horde: dimp/js/src/compose.js,v 1.84.2.34 2009/03/30 22:19:52 slusarz Exp $
  *
- * Copyright 2005-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2005-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -12,8 +12,8 @@
 var DimpCompose = {
     // Variables defaulting to empty/false:
     //   auto_save_interval, button_pressed, compose_cursor, compose_loaded,
-    //   editor_on, mp_padding, resizebcc, resizecc, resizeto, row_height,
-    //   uploading
+    //   dbtext, editor_on, mp_padding, resizebcc, resizecc, resizeto,
+    //   row_height, sbtext, uploading
     last_msg: '',
     textarea_ready: true,
 
@@ -23,57 +23,57 @@ var DimpCompose = {
             if (DIMP.conf_compose.auto_save_interval_val) {
                 DimpCore.doAction('DeleteDraft', { index: $F('index') });
             }
-            if (DIMP.conf_compose.qreply) {
-                this.closeQReply();
-                return true;
-            } else if (DIMP.conf_compose.popup_window) {
-                DimpCore.closePopup();
-            } else {
-                window.location = DIMP.conf.URI_DIMP_INBOX;
-            }
+            return this._closeCompose();
+        }
+    },
+
+    _closeCompose: function()
+    {
+        if (DIMP.conf_compose.qreply) {
+            this.closeQReply();
+        } else if (DIMP.baseWindow || DIMP.conf_compose.popup) {
+            DimpCore.closePopup();
+        } else {
+            DimpCore.redirect(DIMP.conf.URI_DIMP_INBOX);
         }
-        return false;
     },
 
     closeQReply: function()
     {
-        this.last_msg = '';
         var al = $('attach_list').childElements();
+        this.last_msg = '';
+
         if (al.size()) {
             this.removeAttach(al);
         }
-        $('messageCache').setValue('');
-        $('draft_index').setValue('');
+
+        $('draft_index', 'messageCache').invoke('setValue', '');
         $('qreply', 'sendcc', 'sendbcc').invoke('hide');
         [ $('msgData'), $('togglecc').up(), $('togglebcc').up() ].invoke('show');
         if (this.editor_on) {
             this.toggleHtmlEditor();
         }
-        Form.reset('compose');
+        $('compose').reset();
 
         // Disable auto-save-drafts now.
         if (this.auto_save_interval) {
-            window.clearInterval(this.auto_save_interval);
-            this.auto_save_interval = null;
+            this.auto_save_interval.stop();
         }
     },
 
     change_identity: function()
     {
-        var id = $F('identity'),
+        var lastSignature, msg, nextSignature, pos,
+            id = $F('identity'),
             last = this.get_identity($F('last_identity')),
-            lastSignature,
             msgval = $('message'),
-            msg,
             next = this.get_identity(id),
-            nextSignature,
-            pos,
             ssm = $('save_sent_mail');
 
         $('sent_mail_folder_label').setText(next.id[5]);
         $('bcc').setValue(next.id[6]);
         if (ssm) {
-            ssm.checked = next.id[4];
+            ssm.writeAttribute('checked', next.id[4]);
         }
 
         // Finally try and replace the signature.
@@ -96,7 +96,7 @@ var DimpCompose = {
             // http://simonwillison.net/2004/Sep/20/newlines/
             msg = msg.replace(/<p>\s*<!--begin_signature-->[\s\S]*?<!--end_signature-->\s*<\/p>/, lastSignature);
         } else {
-            msg = msgval.value.replace(/\r\n/g, '\n');
+            msg = $F(msgval).replace(/\r\n/g, '\n');
             lastSignature = last.sig;
             nextSignature = next.sig;
         }
@@ -132,14 +132,19 @@ var DimpCompose = {
         }
     },
 
-    get_identity: function(id)
+    get_identity: function(id, editor_on)
     {
-        return { id: DIMP.conf_compose.identities[id], sig: DIMP.conf_compose.identities[id][((this.editor_on) ? 1 : 0)].replace(/^\n/, '') };
+        editor_on = Object.isUndefined(editor_on) ? this.editor_on : editor_on;
+        return {
+            id: DIMP.conf_compose.identities[id],
+            sig: DIMP.conf_compose.identities[id][(editor_on ? 1 : 0)].replace(/^\n/, '')
+        };
     },
 
     uniqueSubmit: function(action)
     {
-        var c = $('compose'), db, sb;
+        var db, params, sb,
+            c = $('compose');
 
         if (DIMP.SpellCheckerObject) {
             DIMP.SpellCheckerObject.resume();
@@ -149,22 +154,30 @@ var DimpCompose = {
             }
         }
 
+        c.setStyle({ cursor: 'wait' });
+
         if (action == 'send_message' || action == 'save_draft') {
-            c.setStyle({ cursor: 'wait' });
             this.button_pressed = true;
 
             switch (action) {
             case 'send_message':
-                sb = $('send_button');
-                if (sb.visible()) {
-                    sb.hide().parentNode.insertBefore(new Element('LABEL', { id: 'send_label' }).insert(DIMP.text_compose.sending), sb);
-            }
-            break;
+                if (($F('subject') == '') &&
+                    !window.confirm(DIMP.text_compose.nosubject)) {
+                    return;
+                }
+
+                if (!this.sbtext) {
+                    sb = $('send_button');
+                    this.sbtext = sb.getText();
+                    sb.setText(DIMP.text_compose.sending);
+                }
+                break;
 
             case 'save_draft':
-                db = $('draft_button');
-                if (db.visible()) {
-                    db.hide().parentNode.insertBefore(new Element('LABEL', { id: 'draft_label' }).insert(DIMP.text_compose.saving), db);
+                if (!this.dbtext) {
+                    db = $('draft_button');
+                    this.dbtext = db.getText();
+                    db.setText(DIMP.text_compose.saving);
                 }
                 break;
             }
@@ -202,13 +215,19 @@ var DimpCompose = {
 
             // Use an AJAX submit here so that we can do javascript-y stuff
             // before having to close the window on success.
-            DimpCore.doAction('*' + DIMP.conf.compose_url, c.serialize(true), [], this.uniqueSubmitCallback.bind(this));
+            params = c.serialize(true);
+            if (!DIMP.baseWindow) {
+                params.nonotify = true;
+            }
+            DimpCore.doAction('*' + DIMP.conf.compose_url, params, [], this.uniqueSubmitCallback.bind(this));
         }
     },
 
     uniqueSubmitCallback: function(r)
     {
-        var d = r.response, db, sb, elt;
+        var elt,
+            d = r.response;
+
         if (!d) {
             return;
         }
@@ -226,17 +245,17 @@ var DimpCompose = {
 
             case 'save_draft':
                 this.button_pressed = false;
-                if (DIMP.conf_compose.popup_window && DIMP.conf_compose.close_draft) {
+                if (DIMP.baseWindow) {
+                    DIMP.baseWindow.DimpBase.pollFolders();
                     DIMP.baseWindow.DimpCore.showNotifications(r.msgs);
-                    DimpCore.closePopup();
-                    return;
+                }
+                if (DIMP.conf_compose.close_draft) {
+                    return this._closeCompose();
                 }
                 break;
 
             case 'send_message':
                 this.button_pressed = false;
-                /* If we loaded compose.php directly (without a base DIMP
-                 * window), then redirect to the base page. */
                 if (DIMP.baseWindow) {
                     if (d.reply_type == 'reply') {
                         DIMP.baseWindow.DimpBase.flag('answered', d.index, d.reply_folder);
@@ -246,22 +265,13 @@ var DimpCompose = {
                         DIMP.baseWindow.DimpBase.createFolder(d.folder);
                     }
 
-                    if (DIMP.conf_compose.popup_window) {
-                        DIMP.baseWindow.DimpCore.showNotifications(r.msgs);
-                        DimpCore.closePopup();
-                        return;
-                    } else if (DIMP.conf_compose.qreply) {
-                        this.closeQReply();
+                    if (d.draft_delete) {
+                        DIMP.baseWindow.DimpBase.pollFolders();
                     }
-                } else if (DIMP.conf_compose.qreply) {
-                    this.closeQReply();
-                } else if (DIMP.conf_compose.popup_window) {
-                    DimpCore.closePopup();
-                    return;
-                } else {
-                    window.location = DIMP.conf.URI_DIMP_INBOX + '#folder:INBOX';
+
+                    DIMP.baseWindow.DimpCore.showNotifications(r.msgs);
                 }
-                break;
+                return this._closeCompose();
 
             case 'add_attachment':
                 this.uploading = false;
@@ -272,14 +282,13 @@ var DimpCompose = {
                 }
                 if (DIMP.conf_compose.attach_limit != -1 &&
                     $('attach_list').childElements().size() > DIMP.conf_compose.attach_limit) {
-                    $('upload').disabled = false;
+                    $('upload').writeAttribute('disabled', false);
                     elt = new Element('DIV', [ DIMP.text_compose.attachment_limit ]);
                 } else {
                     elt = new Element('INPUT', { type: 'file', name: 'file_1' });
                     elt.observe('change', this.uploadAttachment.bind(this));
                 }
-                elt.id = 'upload';
-                $('attach_cell').replaceChild(elt, $('upload_wait'));
+                $('upload_wait').replace(elt.writeAttribute('id', 'upload'));
                 this.resizeMsgArea();
                 break;
             }
@@ -287,18 +296,17 @@ var DimpCompose = {
             this.button_pressed = false;
         }
 
+        $('compose').setStyle({ cursor: null });
+
         // Re-enable buttons if needed.
         if (!this.button_pressed) {
-            db = $('draft_button');
-            sb = $('send_button');
-            if (!sb.visible()) {
-                $('send_label').remove();
-                sb.show();
+            if (this.sbtext) {
+                $('send_button').setText(this.sbtext);
             }
-            if (!db.visible()) {
-                $('draft_label').remove();
-                db.show();
+            if (this.dbtext) {
+                $('draft_button').setText(this.dbtext);
             }
+            this.dbtext = this.sbtext = null;
         }
 
         DimpCore.showNotifications(r.msgs);
@@ -314,7 +322,7 @@ var DimpCompose = {
             DIMP.SpellCheckerObject.resume();
         }
 
-        var msg, oEditor, text;
+        var msg, text;
 
         if (this.editor_on) {
             this.editor_on = false;
@@ -328,8 +336,7 @@ var DimpCompose = {
                 break;
 
             case 'fckeditor':
-                oEditor = FCKeditorAPI.GetInstance('message');
-                text = oEditor.GetHTML();
+                text = FCKeditorAPI.GetInstance('message').GetHTML();
                 $('messageParent').childElements().invoke('hide');
                 $('message').show();
                 break;
@@ -339,27 +346,28 @@ var DimpCompose = {
         } else {
             this.editor_on = true;
             if (!noupdate) {
-                DimpCore.doAction('Text2Html', { text: $('message').value }, [], this.setMessageText.bind(this), { asynchronous: false });
+                DimpCore.doAction('Text2Html', { text: $F('message') }, [], this.setMessageText.bind(this), { asynchronous: false });
             }
 
             switch (DIMP.conf_compose.js_editor) {
             case 'xinha':
+                this._RTELoading('show');
                 xinha_init();
-                Event.observe(window, 'resize', this.resizeXinha.bind(this));
+                this._hideXinhaLoading();
                 break;
 
             case 'fckeditor':
                 oFCKeditor.Height = this.getMsgAreaHeight();
                 // Try to reuse the old fckeditor instance.
                 try {
-                    oEditor = FCKeditorAPI.GetInstance('message');
-                    oEditor.SetHTML($('message').value);
+                    FCKeditorAPI.GetInstance('message').SetHTML($F('message'));
                     $('messageParent').childElements().invoke('show');
                     $('message').hide();
                 } catch (e) {
+                    this._RTELoading('show');
+                    FCKeditor_OnComplete = this._RTELoading.curry('hide');
                     oFCKeditor.ReplaceTextarea();
                 }
-                Event.observe(window, 'resize', this.resizeFCKeditor.bind(this));
                 break;
             }
         }
@@ -367,6 +375,27 @@ var DimpCompose = {
         $('html').setValue(this.editor_on ? 1 : 0);
     },
 
+    _RTELoading: function(cmd)
+    {
+        var o, r;
+        if (!$('rteloading')) {
+            r = new Element('DIV', { id: 'rteloading' }).clonePosition($('messageParent'));
+            $(document.body).insert(r);
+            o = r.viewportOffset();
+            $(document.body).insert(new Element('SPAN', { id: 'rteloadingtxt' }).setStyle({ top: (o.top + 15) + 'px', left: (o.left + 15) + 'px' }).insert(DIMP.text.loading));
+        }
+        $('rteloading', 'rteloadingtxt').invoke(cmd);
+    },
+
+    _hideXinhaLoading: function()
+    {
+        if (_editors && _editors.message) {
+            _editors.message._onGenerate = this._RTELoading.curry('hide');
+        } else {
+            this._hideXinhaLoading.bind(this).delay(.05);
+        }
+    },
+
     toggleHtmlCheckbox: function()
     {
         if (!this.editor_on || window.confirm(DIMP.text_compose.toggle_html)) {
@@ -374,22 +403,9 @@ var DimpCompose = {
         }
     },
 
-    resizeXinha: function()
-    {
-        $('messageParent').select('.htmlarea').first().setStyle({ height: this.getMsgAreaHeight() + 'px' });
-    },
-
-    resizeFCKeditor: function()
-    {
-        $('messageParent').select('iframe').last().setStyle({ height: this.getMsgAreaHeight() + 'px' });
-    },
-
     getMsgAreaHeight: function()
     {
-        // Need an extra 2 pixels of padding since even if $('message') has
-        // no padding, the browser (FF at least) adds a pixel of padding on
-        // the top and bottom of the textarea.
-        return document.viewport.getHeight() - $('messageParent').cumulativeOffset()[1] - this.mp_padding - 2;
+        return document.viewport.getHeight() - $('messageParent').cumulativeOffset()[1] - this.mp_padding;
     },
 
     initializeSpellChecker: function()
@@ -430,12 +446,12 @@ var DimpCompose = {
             DIMP.SpellCheckerObject.htmlArea = DIMP.SpellCheckerObject.htmlAreaParent = null;
             switch (DIMP.conf_compose.js_editor) {
             case 'xinha':
-                _editors.message.setHTML(_editors.message.inwardHtml(_editors.message._textArea.value));
+                _editors.message.setHTML(_editors.message.inwardHtml($F(_editors.message._textArea)));
                 break;
 
             case 'fckeditor':
                 var ed = FCKeditorAPI.GetInstance('message');
-                ed.SetHTML($('message').value);
+                ed.SetHTML($F('message'));
                 ed.Events.AttachEvent('OnAfterSetHTML', function() { this.textarea_ready = true; }.bind(this));
                 break;
             }
@@ -446,8 +462,7 @@ var DimpCompose = {
     {
         var ta = $('message');
         if (!ta) {
-            ta = new Element('TEXTAREA', { id: 'message', name: 'message', style: 'width:100%;' }).insert(r.response.text);
-            $('messageParent').appendChild(ta);
+            $('messageParent').insert(new Element('TEXTAREA', { id: 'message', name: 'message', style: 'width:100%;' }).insert(r.response.text));
         } else {
             ta.setValue(r.response.text);
         }
@@ -466,20 +481,19 @@ var DimpCompose = {
             return;
         }
 
-        var bcc_add,
-            fo,
+        var bcc_add, fo,
             identity = this.get_identity($F('last_identity')),
             msgval = $('message');
 
-        if (this.last_msg != '' &&
-            this.last_msg != msgval.value.replace(/\r/g, '') &&
+        if (!this.last_msg.empty() &&
+            this.last_msg != $F(msgval).replace(/\r/g, '') &&
             !window.confirm(DIMP.text_compose.fillform)) {
             return;
         }
 
         // Set auto-save-drafts now if not already active.
         if (DIMP.conf_compose.auto_save_interval_val && !this.auto_save_interval) {
-            this.auto_save_interval = window.setInterval(function() {
+            this.auto_save_interval = new PeriodicalExecuter(function() {
                 var cur_msg;
                 if (this.editor_on) {
                     switch (DIMP.conf_compose.js_editor) {
@@ -492,14 +506,14 @@ var DimpCompose = {
                         break;
                     }
                 } else {
-                    cur_msg = msgval.value;
+                    cur_msg = $F(msgval);
                 }
                 cur_msg = cur_msg.replace(/\r/g, '');
-                if (cur_msg != '' && this.last_msg != cur_msg) {
+                if (!cur_msg.empty() && this.last_msg != cur_msg) {
                     this.uniqueSubmit('auto_save_draft');
                     this.last_msg = cur_msg;
                 }
-            }.bind(this), DIMP.conf_compose.auto_save_interval_val * 60000);
+            }.bind(this), DIMP.conf_compose.auto_save_interval_val * 60);
         }
 
         if (this.editor_on) {
@@ -519,7 +533,7 @@ var DimpCompose = {
         } else {
             msgval.setValue(msg);
             this.setCursorPosition(msgval);
-            this.last_msg = msgval.value.replace(/\r/g, '');
+            this.last_msg = $F(msgval).replace(/\r/g, '');
         }
 
         $('to').setValue(header.to);
@@ -536,7 +550,7 @@ var DimpCompose = {
             this.resizebcc.resizeNeeded();
         }
         if (identity.id[6]) {
-            bcc_add = $('bcc').value;
+            bcc_add = $F('bcc');
             if (bcc_add) {
                 bcc_add += ', ';
             }
@@ -576,17 +590,16 @@ var DimpCompose = {
                 break;
             }
         } catch (e) {
-            this.focusEditor.bind(this).delay(0.1);
+            this.focusEditor.bind(this).defer();
         }
     },
 
     addAttach: function(number, name, type, size)
     {
         var div = new Element('DIV').insert(name + ' [' + type + '] (' + size + ' KB) '),
-
             input = new Element('INPUT', { type: 'button', atc_id: number, value: DIMP.text_compose.remove });
-        div.appendChild(input);
-        $('attach_list').appendChild(div);
+        div.insert(input);
+        $('attach_list').insert(div);
         input.observe('click', this.removeAttach.bind(this, [ input.up() ]));
         this.resizeMsgArea();
     },
@@ -605,19 +618,38 @@ var DimpCompose = {
 
     resizeMsgArea: function()
     {
-        if (!this.compose_loaded || this.editor_on) {
+        var m, rows,
+            de = document.documentElement,
+            msg = $('message');
+
+        if (!this.compose_loaded) {
+            return;
+        }
+
+        if (this.editor_on) {
+            switch (DIMP.conf_compose.js_editor) {
+            case 'xinha':
+                m = $('messageParent').select('.htmlarea').first();
+                break;
+
+            case 'fckeditor':
+                m = $('messageParent').select('iframe').last();
+                break;
+            }
+            if (m) {
+                m.setStyle({ height: this.getMsgAreaHeight() + 'px' });
+            } else {
+                this.resizeMsgArea.bind(this).defer();
+            }
             return;
         }
 
-        var m, msg = $('message');
         this.mp_padding = $('messageParent').getHeight() - msg.getHeight();
 
         if (!this.row_height) {
-            m = msg.cloneNode(false);
             // Change the ID and name to not conflict with msg node.
-            m.id = m.name = null;
-            m.setStyle({ visibility: 'hidden' });
-            document.body.appendChild(m);
+            m = $(msg.cloneNode(false)).writeAttribute({ id: null, name: null }).setStyle({ visibility: 'hidden' });
+            $(document.body).insert(m);
             m.writeAttribute('rows', 1);
             this.row_height = m.getHeight();
             m.writeAttribute('rows', 2);
@@ -628,26 +660,26 @@ var DimpCompose = {
         /* Logic: Determine the size of a given textarea row, divide that size
          * by the available height, round down to the lowest integer row, and
          * resize the textarea. */
-        msg.writeAttribute('rows', parseInt(this.getMsgAreaHeight() / this.row_height) - 1);
-        // IE 7 needs this.
-        msg.disabled = false;
+        rows = parseInt(this.getMsgAreaHeight() / this.row_height);
+        msg.writeAttribute({ rows: rows, disabled: false });
+        if (de.scrollHeight - de.clientHeight) {
+            msg.writeAttribute({ rows: rows - 1 });
+        }
     },
 
     uploadAttachment: function()
     {
-        var div, u = $('upload');
+        var u = $('upload');
         $('submit_frame').observe('load', this.attachmentComplete.bind(this));
         this.uniqueSubmit('add_attachment');
-        u.stopObserving('change');
-        div = new Element('DIV', { id: 'upload_wait' }).insert(DIMP.text_compose.uploading + ' ' + u.value);
-        $('attach_cell').replaceChild(div, u);
+        u.stopObserving('change').replace(new Element('DIV', { id: 'upload_wait' }).insert(DIMP.text_compose.uploading + ' ' + $F(u)));
     },
 
     attachmentComplete: function()
     {
-        var doc, sf = $('submit_frame');
+        var sf = $('submit_frame'),
+            doc = sf.contentDocument || sf.contentWindow.document;
         sf.stopObserving('load');
-        doc = sf.contentDocument || sf.contentWindow.document;
         DimpCore.doActionComplete({ responseText: doc.body.innerHTML }, this.uniqueSubmitCallback.bind(this));
     },
 
@@ -660,21 +692,20 @@ var DimpCompose = {
     /* Sets the cursor to the given position. */
     setCursorPosition: function(input)
     {
-        var msg, pos, range;
+        var pos, range;
 
         switch (DIMP.conf_compose.compose_cursor) {
         case 'top':
             pos = 0;
-            msg = $('message');
-            msg.setValue('\n' + msg.value);
+            $('message').setValue('\n' + $F('message'));
             break;
 
         case 'bottom':
-            pos = $('message').value.length;
+            pos = $F('message').length;
             break;
 
         case 'sig':
-            pos = $('message').value.replace(/\r\n/g, '\n').lastIndexOf(this.get_identity($F('last_identity')).sig) - 1;
+            pos = $F('message').replace(/\r\n/g, '\n').lastIndexOf(this.get_identity($F('last_identity')).sig) - 1;
             break;
 
         default:
@@ -701,9 +732,9 @@ var DimpCompose = {
     {
         window.open(DIMP.conf_compose.abook_url, 'contacts', 'toolbar=no,location=no,status=no,scrollbars=yes,resizable=yes,width=550,height=300,left=100,top=100');
     }
-};
+},
 
-var ResizeTextArea = Class.create({
+ResizeTextArea = Class.create({
     // Variables defaulting to empty:
     //   defaultRows, field, onResize
     maxRows: 5,
@@ -716,17 +747,17 @@ var ResizeTextArea = Class.create({
         this.onResize = onResize;
 
         var func = this.resizeNeeded.bindAsEventListener(this);
-        this.field.observe("click", func);
-        this.field.observe("keyup", func);
+        this.field.observe('mousedown', func);
+        this.field.observe('keyup', func);
 
         this.resizeNeeded();
     },
 
     resizeNeeded: function()
     {
-        var lines = this.field.getValue().split('\n'),
+        var lines = $F(this.field).split('\n'),
             cols = this.field.readAttribute('cols'),
-            newRows = lines.size();
+            newRows = lines.size(),
             oldRows = this.field.readAttribute('rows');
 
         lines.each(function(line) {
@@ -748,8 +779,7 @@ var ResizeTextArea = Class.create({
 document.observe('dom:loaded', function() {
     var DC = DimpCompose,
         boundResize = DC.resizeMsgArea.bind(DC),
-        C = DimpCore.clickObserveHandler,
-        sf;
+        C = DimpCore.clickObserveHandler;
 
     DC.initializeSpellChecker();
     $('upload').observe('change', DC.uploadAttachment.bind(DC));
@@ -763,10 +793,7 @@ document.observe('dom:loaded', function() {
     // it will open content in a new window.  See:
     //   http://blog.caboo.se/articles/2007/4/2/ajax-file-upload
     if (Prototype.Browser.WebKit) {
-        sf = $('submit_frame');
-        sf.writeAttribute({ position: 'absolute', width: '1px', height: '1px' });
-        sf.setStyle({ left: '-999px' });
-        sf.show();
+        $('submit_frame').writeAttribute({ position: 'absolute', width: '1px', height: '1px' }).setStyle({ left: '-999px' }).show();
     }
 
     /* Attach click handlers. */
@@ -779,7 +806,7 @@ document.observe('dom:loaded', function() {
         C({ d: $('toggle' + a), f: DC.toggleCC.bind(DC, a) });
     });
     if ($('htmlcheckbox')) {
-        C({ d: $('htmlcheckbox'), f: DC.toggleHtmlCheckbox.bind(DC) });
+        C({ d: $('htmlcheckbox'), f: DC.toggleHtmlCheckbox.bind(DC), ns: true });
     }
     if ($('compose_specialchars')) {
         C({ d: $('compose_specialchars'), f: function() { window.open(DIMP.conf_compose.specialchars_url, 'chars', 'height=220,width=400'); } });
@@ -789,6 +816,9 @@ document.observe('dom:loaded', function() {
         C({ d: a, f: DC.openAddressbook.bind(DC) });
     });
 
+    /* Only allow submit through send button. */
+    $('compose').observe('submit', Event.stop);
+
     /* Attach other handlers. */
     $('identity').observe('change', DC.change_identity.bind(DC));
 
@@ -797,7 +827,6 @@ document.observe('dom:loaded', function() {
     $('togglecc').observe('click', boundResize);
     $('togglebcc').observe('click', boundResize);
     Event.observe(window, 'resize', boundResize);
-
 });
 
 Event.observe(window, 'load', function() {
