@@ -1,8 +1,8 @@
 <?php
 /**
- * $Horde: dimp/lib/DIMP.php,v 1.110.2.19 2008/05/23 07:15:37 slusarz Exp $
+ * $Horde: dimp/lib/DIMP.php,v 1.110.2.38 2009/04/07 04:52:58 slusarz Exp $
  *
- * Copyright 2005-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2005-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -65,7 +65,6 @@ class DIMP {
         // Need to include script files before we start output
         Horde::addScriptFile('prototype.js', 'horde', true);
         Horde::addScriptFile('effects.js', 'imp', true);
-        Horde::addScriptFile('controls.js', 'imp', true);
 
         // ContextSensitive must be loaded before DimpCore.
         while (list($key, $val) = each($scripts)) {
@@ -95,16 +94,26 @@ class DIMP {
 
         echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "DTD/xhtml1-transitional.dtd">' . "\n" .
              (!empty($GLOBALS['language']) ? '<html lang="' . strtr($GLOBALS['language'], '_', '-') . '"' : '<html') . ">\n".
-             "<head>\n" .
-             '<title>' . htmlspecialchars($page_title) . "</title>\n" .
+             "<head>\n";
+
+        // TODO: Make dimp work with IE 8 standards mode
+        if ($GLOBALS['browser']->isBrowser('msie') &&
+            ($GLOBALS['browser']->getMajor() == 8)) {
+            echo '<meta http-equiv="X-UA-Compatible" content="IE=EmulateIE7" />' . "\n";
+        }
+
+        echo '<title>' . htmlspecialchars($page_title) . "</title>\n" .
              '<link href="' . $GLOBALS['registry']->getImageDir() . "/favicon.ico\" rel=\"SHORTCUT ICON\" />\n".
              IMP::wrapInlineScript(DIMP::_includeDIMPJSVars());
 
-        IMP::includeScriptFiles();
         IMP::includeStylesheetFiles(true, 'dimp');
-        IMP::outputInlineScript();
 
         echo "</head>\n";
+
+        // Send what we have currently output so the browser can start
+        // loading CSS/JS. See:
+        // http://developer.yahoo.com/performance/rules.html#flush
+        flush();
     }
 
     /**
@@ -131,25 +140,31 @@ class DIMP {
             $app_urls[$app] = Horde::url($registry->getInitialPage($app), true);
         }
 
+        require DIMP_BASE . '/config/portal.php';
+        foreach ($dimp_block_list as $block) {
+            if (is_a($block['ob'], 'Horde_Block')) {
+                $app = $block['ob']->getApp();
+                if (empty($app_urls[$app])) {
+                    $app_urls[$app] = Horde::url($registry->getInitialPage($app), true);
+                }
+            }
+        }
+
         /* Variables used in core javascript files. */
         $code['conf'] = array(
             'URI_DIMP_INBOX' => Horde::url($dimp_webroot . '/index.php', true, -1),
             'URI_IMP' => Horde::url($dimp_webroot . '/imp.php', true, -1),
-            'URI_PREFS' => Horde::url($horde_webroot . '/services/prefs/', true),
+            'URI_PREFS' => Horde::url($horde_webroot . '/services/prefs/', true, -1),
             'SESSION_ID' => defined('SID') ? SID : '',
 
             'app_urls' => $app_urls,
             'timeout_url' => Auth::addLogoutParameters($horde_webroot . '/login.php', AUTH_REASON_SESSION),
             'message_url' => Horde::url($dimp_webroot . '/message.php'),
             'compose_url' => Horde::url($dimp_webroot . '/compose.php'),
-            'prefs_url' => Horde::getServiceLink('options', 'dimp'),
+            'prefs_url' => str_replace('&amp;', '&', Horde::getServiceLink('options', 'dimp')),
 
-            'sortfrom' => SORTFROM,
-            'sortsubject' => SORTSUBJECT,
             'sortthread' => SORTTHREAD,
             'sortdate' => SORTDATE,
-            'sortarrival' => SORTARRIVAL,
-            'sortsize' => SORTSIZE,
 
             'idx_sep' => IMP_IDX_SEP,
 
@@ -159,7 +174,9 @@ class DIMP {
             'forward_default' => $prefs->getValue('forward_default'),
             'spam_folder' => IMP::folderPref($prefs->getValue('spam_folder'), true),
             'spam_reporting' => (int) !empty($conf['spam']['reporting']),
+            'spam_spamfolder' => (int) !empty($conf['spam']['spamfolder']),
             'ham_reporting' => (int) !empty($conf['notspam']['reporting']),
+            'ham_spamfolder' => (int) !empty($conf['notspam']['spamfolder']),
             'refresh_time' => (int) $prefs->getValue('refresh_time'),
             'search_all' => (int) !empty($conf['search']['search_all']),
 
@@ -179,8 +196,6 @@ class DIMP {
             'login_view' => $dimp_prefs->getValue('login_view'),
             'background_inbox' => !empty($dimp_conf['viewport']['background_inbox']),
 
-            'show_quota' => (isset($_SESSION['imp']['quota']) && is_array($_SESSION['imp']['quota'])),
-
             // Turn debugging on?
             'debug' => !empty($dimp_conf['js']['debug']),
         );
@@ -191,24 +206,18 @@ class DIMP {
             'prefs' => _("User Options"),
             'search' => _("Search"),
             'resfound' => _("results found"),
+            'message' => _("Message"),
             'messages' => _("Messages"),
             'of' => _("of"),
             'nomessages' => _("No Messages"),
             'hidetext' => _("Hide Quoted Text"),
             'showtext' => _("Show Quoted Text"),
             'lines' => _("lines"),
-            'hideaddr' => _("Hide Addresses"),
-            'showaddr' => _("Show Addresses"),
-            'recips' => _("recipients"),
             'ok' => _("Ok"),
+            'moveto' => _("Move %s to %s"),
+            'baselevel' => _("base level of the folder tree"),
             'cancel' => _("Cancel"),
             'loading' => _("Loading..."),
-            'totext' => _("To"),
-            'from' => _("From"),
-            'thread' => _("Thread"),
-            'subject' => _("Subject"),
-            'datesort' => _("Date"),
-            'arrival' => _("Arrival"),
             'check' => _("Checking..."),
             'getmail' => _("Get Mail"),
             'ajax_timeout' => _("There has been no contact with the remote server for several minutes. The server may be temporarily unavailable or network problems may be interrupting your session. You will not see any updates until the connection is restored."),
@@ -244,6 +253,7 @@ class DIMP {
             /* Gettext strings used in compose page. */
             $code['text_compose'] = array_map('addslashes', array(
                 'cancel' => _("Cancelling this message will permanently discard its contents and will delete auto-saved drafts.\nAre you sure you want to do this?"),
+                'nosubject' => _("The message does not have a Subject entered.") . "\n" . _("Send message without a Subject?"),
                 'fillform' => _("You have already changed the message body, are you sure you want to drop the changes?"),
                 'remove' => _("remove"),
                 'uploading' => _("Uploading..."),
@@ -282,6 +292,15 @@ class DIMP {
         return 'DimpCore.showNotifications(' . Horde_Serialize::serialize($msgs, SERIALIZE_JSON) . ')';
     }
 
+    /**
+     * Output the javascript for the page.
+     */
+    function outputJS()
+    {
+        IMP::includeScriptFiles();
+        IMP::outputInlineScript();
+    }
+
     /**
      * Formats the response to send to javascript code when dealing with
      * folder operations.
@@ -336,32 +355,37 @@ class DIMP {
      *
      * @return stdClass  The element object. Contains the following items:
      * <pre>
-     * 'ch' (children) = Does the folder contain children?
-     * 'cl' (class) = The CSS class.
-     * 'co' (container) = Is this folder a container element?
-     * 'i' (icon) = A user defined icon to use.
-     * 'l' (label) = The folder display label.
-     * 'm' (mbox) = The mailbox value.
-     * 'pa' (parent) = The parent element.
-     * 'po' (polled) = Is the element polled?
-     * 's' (special) = Is this a "special" element?
-     * 'u' (unseen) = The number of unseen messages.
+     * 'ch' (children) = Does the folder contain children? [boolean]
+     *                   [DEFAULT: no]
+     * 'cl' (class) = The CSS class. [string] [DEFAULT: 'base']
+     * 'co' (container) = Is this folder a container element? [boolean]
+     *                    [DEFAULT: no]
+     * 'i' (icon) = A user defined icon to use. [string] [DEFAULT: none]
+     * 'l' (label) = The folder display label. [string]
+     * 'm' (mbox) = The mailbox value. [string]
+     * 'pa' (parent) = The parent element. [string]
+     * 'po' (polled) = Is the element polled? [boolean] [DEFAULT: no]
+     * 's' (special) = Is this a "special" element? [boolean] [DEFAULT: no]
+     * 'u' (unseen) = The number of unseen messages. [integer]
      * </pre>
      */
     function _createFolderElt($elt)
     {
         $ob = new stdClass;
-        $ob->ch = $elt['children'];
-        $ob->cl = 'base';
-        $ob->co = $elt['container'];
-        $ob->i = '';
+        if ($elt['children']) {
+           $ob->ch = 1;
+        }
         $ob->l = $elt['base_elt']['l'];
         $ob->m = rawurlencode($elt['value']);
         $ob->pa = rawurlencode($elt['parent']);
-        $ob->po = $elt['polled'];
-        $ob->s = false;
+        if ($elt['polled']) {
+            $ob->po = 1;
+        }
 
-        if (!$ob->co) {
+        if ($elt['container']) {
+            $ob->co = 1;
+            $ob->cl = 'exp';
+        } else {
             if ($elt['polled']) {
                 $ob->u = $elt['unseen'];
             }
@@ -369,27 +393,27 @@ class DIMP {
             switch ($elt['special']) {
             case IMPTREE_SPECIAL_INBOX:
                 $ob->cl = 'inbox';
-                $ob->s = true;
+                $ob->s = 1;
                 break;
 
             case IMPTREE_SPECIAL_TRASH:
                 $ob->cl = 'trash';
-                $ob->s = true;
+                $ob->s = 1;
                 break;
 
             case IMPTREE_SPECIAL_SPAM:
                 $ob->cl = 'spam';
-                $ob->s = true;
+                $ob->s = 1;
                 break;
 
             case IMPTREE_SPECIAL_DRAFT:
                 $ob->cl = 'drafts';
-                $ob->s = true;
+                $ob->s = 1;
                 break;
 
             case IMPTREE_SPECIAL_SENT:
                 $ob->cl = 'sent';
-                $ob->s = true;
+                $ob->s = 1;
                 break;
 
             default:
@@ -405,8 +429,6 @@ class DIMP {
                 }
                 break;
             }
-        } else {
-            $ob->cl = 'exp';
         }
 
         if ($elt['user_icon']) {
@@ -420,18 +442,27 @@ class DIMP {
     /**
      * Returns a stdClass response object with added notification information.
      *
-     * @param string $data   The 'response' data.
-     * @param boolean $auto  If true, DimpCore will automatically display the
-     *                       notification.  If false, the callback handler
-     *                       is responsible for displaying the notification.
+     * @param string $data     The 'response' data.
+     * @param boolean $notify  If true, adds notification information to
+     *                         object.
+     * @param boolean $auto    If true, DimpCore will automatically display the
+     *                         notification.  If false, the callback handler
+     *                         is responsible for displaying the notification.
      */
-    function getNotificationResponse($data = null, $auto = true)
+    function prepareResponse($data = null, $notify = true, $auto = true)
     {
         $response = new stdClass();
         $response->response = $data;
-        $GLOBALS['notification']->notify(array('listeners' => 'status'));
-        $response->msgs = $GLOBALS['dimp_listener']->getStack();
-        $response->msgs_auto = (bool)$auto;
+        if ($notify) {
+            $GLOBALS['notification']->notify(array('listeners' => 'status'));
+            $stack = $GLOBALS['dimp_listener']->getStack();
+            if (!empty($stack)) {
+                $response->msgs = $GLOBALS['dimp_listener']->getStack();
+                if (!(bool)$auto) {
+                    $response->msgs_noauto = true;
+                }
+            }
+        }
         return $response;
     }
 
